"""
Patch 'MultiHook' for downgrades from Zope 2.5 to Zope 2.4/2.3.
Zope 2.5 inserts an instance of 'SessionTraverser' as a before publishing
traverse function into the Application object's root.  When Zope 2.4 attempts
to use this as a before publishing traverse method, an error is raised because
the MultiHook is actually trying to call a 'broken' object (because there
is no such class 'SessionTraverser' in Zope 2.4).  As 'broken' objects
have no __call__ method, the symptom shows itself as an 'AttributeError:
__call__ in line 171 of BeforeTraverse.py.

This patch causes the error to be logged but not raised to the application
layer.
"""

def patchMultiHook():
    global MultiHook
    class MultiHook:
        """Class used to multiplex hook.

        MultiHook calls the named hook from the class of the container, then
        the prior hook, then all the hooks in its list.
        """
        def __init__(self, hookname, prior, defined_in_class):
            self._hookname = hookname
            self._prior = prior
            self._defined_in_class = defined_in_class
            self._list = []

        def __call__(self, container, request):
            if self._defined_in_class:
                # Assume it's an unbound method
                getattr(container.__class__, self._hookname)(container,request)
            prior = self._prior
            if prior is not None:
                prior(container, request)
            for cob in self._list:
                try:
                    cob(container, request)
                except (ArithmeticError, AttributeError, FloatingPointError,
                        IOError, ImportError, IndexError, KeyError,
                        OSError, OverflowError, TypeError, ValueError,
                        ZeroDivisionError):
                    # Only catch exceptions that are likely to be logic errors.
                    # We shouldn't catch Redirects, Unauthorizeds, etc. since
                    # the programmer may want to raise them deliberately.
                    from zLOG import LOG, ERROR
                    import sys
                    msg = ('Error while invoking hook %s on %s' %
                           (self._hookname, container))
                    LOG('BeforeTraverse', ERROR, msg, error=sys.exc_info())

        def add(self, cob):
            self._list.append(cob)

from ZPublisher import BeforeTraverse
exec patchMultiHook.func_code in BeforeTraverse.__dict__
